proc loadali {object file {skeletonfile {}}} {
	if {"$file"==""} return
	private $object loadali canvas current
	set undostatus [$canvas undo status]
	$canvas undo 0
	set f [open $file]
	set ids ""
	set helixseq ""
	set maskseq ""
	gets $f;gets $f;gets $f;gets $f
	while 1 {
		set line [gets $f]
		if {"$line"==""} break
		if ![regexp {^(.*) ([0-9A-Z ])[0-9 ][0-9 ][0-9 ] ([^ ].*)$} $line temp sequence type id] {
			error "Error reading file"
		}
		set id [string trimright $id]
		if {"$type"=="H"} {
			set helixseq $sequence
		} elseif {"$type"=="M"} {
			set maskseq $sequence
		} else {
			set seq($id) $sequence
			lappend ids $id
		}
	}
	close $f
	if {"$helixseq"==""} {
		foreach id $ids {
			if [regexp {^Helix numbering} $id] {
				set helixseq $seq($id)
				set ids [list_remove $ids $id]
			}
		}
	}
	if {"$helixseq"==""} {error "no Helix numbering found"}
	if {"$maskseq"==""} {
		foreach id $ids {
			if [regexp {^mask} $id] {
				set maskseq $seq($id)
				set ids [list_remove $ids $id]
			}
		}
	}
	if {"$ids"==""} {error "no sequences found"}
	# Select when multiple sequences are present
	# ------------------------------------------
	if [info exists loadali] {unset loadali}
	if {[llength $ids]>1} {
		Classy::SelectDialog .loadaliselect -title "Select structures" \
		-help multiple_select -selectmode multiple -command [list set [privatevar $object loadali(list)]]
		.loadaliselect fill $ids
		tkwait window .loadaliselect
		set ids $loadali(list)
	}
	if {[llength $ids]>1} {
		Classy::Dialog .loadali -title "Multiple entries" -closecommand {set loadali(stop) 1} -help multiple_entries
		.loadali add go "Go" {destroy .loadali} default
		Classy::OptionBox .loadali.options.opt -label "File contains multiple entries" \
			-variable loadali(opt) -orient vertical
		.loadali.options.opt add batch "Batch job: one structure per file" -command {
			set loadali(file) [Classy::savefile -title "Save files as" -help multiple_saveas]
		}
		.loadali.options.opt add load "Load selected structures on current page"
		pack .loadali.options.opt -fill x
		set loadali(opt) load
		tkwait window .loadali
		if [info exists loadali(stop)] {
			unset loadali
			return
		}
	
	} else {
		set loadali(opt) load
	}
	Classy::busy $object
	foreach id $ids {
		if {"$loadali(opt)"=="batch"} {
			clearpage
		}
		if {"$maskseq"==""} {
			set error [catch {set data [struct_parse $seq($id) $helixseq]} result]
		} else {
			set error [catch {set data [struct_parse -tagsequence $maskseq $seq($id) $helixseq]} result]
		}
		if $error {
			regsub {H} $result {} result
			error "$result\n(remember that the RnaViz structure parser is stricter than that in DCSE v3.x)"
		}
		createstruct $object $id $data $skeletonfile
		if {"$loadali(opt)"=="batch"} {
			savestruct [file root $loadali(file)]$id[file extension $loadali(file)]
		}
	}
	unset seq
	Classy::busy remove
	$canvas undo $undostatus
}


