# Struct creation of canvas objects and info array
# ----------------------------------------------------------------------
# codes
# order: base pos basetags name type part side nonst
# name:
# 	H<name>
# 	H<name>'
#	S<name>
# type:
# 	d:double stranded
# 	l:loop
# 	s:single stranded
# part:
# 	d<num>
#	s<num>
#	s<num>'
# side:
# 	p5: 5 prime side
# 	p3: 3 prime side
# nonst
# 	s
#	ns

proc createfromdata {object drawing data} {
	private $object canvas drawinfo
	set keepundo [$canvas undo status]
	$canvas undo off
	set w [Classy::window $canvas]
	set pos 1
	set ys [expr {$drawinfo($drawing,ys)*2}]
	set dotwidth $drawinfo($drawing,dotwidth)
	set nsdotwidth $drawinfo($drawing,nsdotwidth)
	set basefont $drawinfo($drawing,basefont)
	set helixfont $drawinfo($drawing,helixfont)
	set temp [list_subindex $data 0]
	set temp [list_remove $temp {} start end]
	set drawinfo($drawing,sequence) $temp
	set index [$canvas create text 50 100 -text "o" -tags "$drawing {_ start} _str" -fill blue -font $basefont]
	set drawinfo($drawing,start) $index
	set actbasefont [$canvas _font $basefont]
	
	set elementlist ""
	set lastname ""
	set dotlist ""
	set len [llength $data]
	busy $object "Creating structure" [expr {1.4*$len}]
	foreach char $data {
		foreach {base pos basetags name type part side nonst} $char {break}
		if {("$type"=="d")||("$type"=="l")} {
			regsub {^[^0-9]+} $part {} part
			regsub {'$} $part {} part
			if {"$type" == "l"} {incr part}
			if {"$side" == "p5"} {
				if {"$type" == "d"} {
					if {"$nonst"!="ns"} {
						lappend dotlist [list $drawing [list _ dot $name $part] _$name _dot _str]
					} else {
						lappend dotlist [list $drawing [list _ dotns $name $part] _$name _dotns _dot _str]
					}
					lappend drawinfo($drawing,$name,dots) $part
				}
				lappend drawinfo($drawing,$name,objs) [list $type $part $side]
			} else {
				set name ${name}'
				lappend drawinfo($drawing,$name,objs) [list $type $part $side]
			}
			set tag [list _ $type $name $part $side]
		} elseif {"$type"=="s"} {
			set tag [list _ $type $name]
		} else {
			error "Wrong type"
		}
		incr pos
		set tags [list_concat [list $drawing $tag _$name _base _str] $basetags]
		if {"$name"!="$lastname"} {
			lappend elementlist $name
			set lastname $name
		}
		set prevside $side
		set index [$w create text 10 10 -text $base -tags $tags -font $actbasefont]
		lappend drawinfo($drawing,$name,ids) $index
		busyincr $object
	}
	
	# Draw dots
	# ----------------------------
	foreach item $dotlist {
		if {"[lindex $item 3]" == "_dot"} {
			set width $dotwidth
			set ns 0
		} else {
			set width $nsdotwidth
			set ns 1
		}
		set tag [lindex $item 1]
		set index [$w create line 100 100 100 101 -tags $item -capstyle round -width $width]
		set name [lindex [lindex $item 1] 2]
		lappend drawinfo($drawing,$name,dotids) $index
		if {$ns&&!$drawinfo($drawing,nsshow)} {
			$canvas lower $index _paper
		}
		busyincr $object
	}
	# Create endpoint
	set index [$canvas create text 50 [expr {100+2*$ys}] -text "o" -tags [list $drawing {_ end} _str] -fill blue -font $basefont]
	set drawinfo($drawing,end) $index
#	lappend objectlist end
#	lappend idlist $index
	# Create helix names
	set drawinfo($drawing,names) ""
	foreach item $elementlist {
		if [regexp {^H(.*)'$} $item temp helix] {
			lappend drawinfo($drawing,names) ""
		} elseif [regexp {^H(.*)} $item temp helix] {
			set index [$canvas create text 100 100 -text $helix -tags [list $drawing [list _ name ${item}] _$item _helixname _str _text] -font $helixfont]
			lappend drawinfo($drawing,names) $index
			lappend drawinfo($drawing,$item,lids) $index
		} else {
			lappend drawinfo($drawing,names) ""
		}
	}
#	set drawinfo($drawing,objectlist) $objectlist
#	set drawinfo($drawing,idlist) $idlist
	set drawinfo($drawing,elementlist) $elementlist
	set prev start
	busyincr $object
	foreach el $elementlist next [lrange $elementlist 1 end] {
		if {[regexp ^H $el]&&[regexp ^H $prev]} {
			regsub ^H $prev S prev
		}
		set drawinfo($drawing,prv,$el) $prev
		if {[regexp ^H $el]&&[regexp ^H $next]} {
			regsub ^H $el S next
		}
		set drawinfo($drawing,nxt,$el) $next
		set prev $el
	}
	set drawinfo($drawing,nxt,$el) end
	notbusy $object
	$canvas undo $keepundo
	return $elementlist
}

proc createstruct {object seqname data {skeletonfile {}}} {
	private $object canvas drawinfo canvas
	update idletasks
	# Some tests first
	set prev {}
	catch {unset test}
	foreach item [list_subindex $data 3] type [list_subindex $data 4] {
		if {[string_equal $type s]} {
			set prev {}
			continue
		}
		if {![string_equal $type d]} {
			continue
		}
		if {[string_equal $item $prev]} {
			incr test($item)
		} else {
			if [info exists test($item)] {
				set test($item') $test($item)
				set test($item) 1
			} else {
				set test($item) 1
			}
			set prev $item
		}
	}
	set error {}
	foreach item [list_remdup [array names test]] {
		set item [string trimright $item ']
		if {[get test($item) 0] != [get test($item') 0]} {
			append error "error in helix \"[string range $item 1 end]\": number of bases does not match\n"
		}
	}
	if {[string length $error]} {error $error}
	catch {unset test}
	# Tests were ok, create
	set drawing [$canvas group withtag {}]
	foreach name [array names drawinfo $drawing,*] {
		unset drawinfo($name)
	}
	setsettings $object $drawing
	lappend drawinfo(drawings) $drawing
	set zoom [$canvas zoom]
	$canvas zoom 1

	# Initialise drawing from data
	# ----------------------------
	
	createfromdata $object $drawing $data
	if {"$skeletonfile"!=""} {
		loadskeleton $object $drawing $skeletonfile -redrawhelices 1
	} else {
		set dist $drawinfo($drawing,zoom_dist)
		set elementlist $drawinfo($drawing,elementlist)
		busy $object "Drawing elements" [expr {[llength $elementlist]+1}]
		update
		set loop [findloop all $elementlist]
		set size [findloopsize $object $drawing $loop]
		foreach {startx starty} [$canvas coords $drawinfo($drawing,start)] {}
		foreach {endx endy} [$canvas coords $drawinfo($drawing,end)] {}
		foreach {centerx centery} [findcenter $startx $starty $endx $endy [expr {($size+1)*$dist}]] {}
		set points [centercirclepoints $size $startx $starty $endx $endy $centerx $centery]
		drawitems $object $drawing $loop $points
		foreach item $elementlist {
			if [regexp {^H.*[^']$} $item] {
				set loop [findloop $item $elementlist]
				set size [findloopsize $object $drawing $loop]
				set start [$canvas coords [lindex $drawinfo($drawing,$item,ids) end]]
				set end [$canvas coords [lindex $drawinfo($drawing,$item',ids) 0]]
				foreach {startx starty} $start {}			
				foreach {endx endy} $end {}			
				set temp [expr {($size+1)*$dist}]
				set temp2 [expr {2.4*[distance $startx $starty $endx $endy]}]
				if {$temp<$temp2} {set temp $temp2}
				set centerx [findcenter $startx $starty $endx $endy $temp]
				set centery [list_pop centerx]
				set points [centercirclepoints $size $startx $starty $endx $endy $centerx $centery]
				drawitems $object $drawing $loop $points
			}
			busyincr $object
		}
	}
	set font $drawinfo($drawing,helixfont)
	set bbox [$canvas bbox $drawing]
	$canvas create text [lindex $bbox 0] [expr {[lindex $bbox 1]-10}] -text $seqname -tags "$drawing {_ label} _text" -font $font
	$canvas zoom $zoom
	notbusy $object
}

proc findloop {name elementlist} {
	if {"$name"=="all"} {
		set list $elementlist
	} else {
		set pos [lsearch $elementlist $name]
		if {$pos == -1} {return ""}
		set pos2 [lsearch $elementlist $name']
		if {$pos2 == -1} {error "error in structure: \"$name'\" not found"}
		set list [lrange $elementlist [expr {$pos+1}] [expr {$pos2-1}]]
	}
	set loop ""
	while 1 {
		if {"$list"==""} {break}
		set elem [list_shift list]
		if [regexp {^S} $elem] {
			lappend loop $elem
		} else {
			regsub {'$} $elem {} n
			lappend loop $n
			set pos [lsearch -exact $list $elem']
			if {$pos != -1} {
				set list [lrange $list [expr {$pos+1}] end]
			}
		}
	}
	return $loop
}

proc findloopsize {object drawing loop} {
	private $object drawinfo
	set size 0
	foreach elem $loop {
		if [regexp {^S} $elem] {
			incr size [llength [get drawinfo($drawing,$elem,ids) ""]]
		} else {
			incr size 2
		}
	}
	return $size
}

proc drawitems {object drawing loop points} {
#puts [list drawitems $object $drawing $loop]
#putsvars object drawing loop points
	private $object drawinfo canvas
	set ys [expr {$drawinfo($drawing,ys)*2}]
	set lagging 0
	foreach name $loop {
		if [regexp {^S} $name] {
			set lagging 1
			set lname $name
			set items [getitem $object $drawing $name]
			set num [llength $items]
			if {$num == 0} {
				set lx 0
				set ly 0
			} else {
				set pos [expr {(($num-1)/2)*2}]
				set lx [lindex $points $pos]
				incr pos
				set ly [lindex $points $pos]
				set points [lrange $points [expr {$num*2}] end]
			}
		} else {
			set x1 [list_shift points]
			set y1 [list_shift points]
			set x2 [list_shift points]
			set y2 [list_shift points]
			set angle [sqangle $x1 $y1 $x2 $y2]
			set dist [distance $x1 $y1 $x2 $y2]
			set diff [expr {($ys-$dist)/2}]
			if {$diff!=0} {
				set stx [expr {$x1-($diff/$dist)*($x2-$x1)}]
				set sty [expr {$y1-($diff/$dist)*($y2-$y1)}]
				drawhelix $object $drawing $name $stx $sty [expr {$angle+90}]
			} else {
				drawhelix $object $drawing $name $x1 $y1 [expr {$angle+90}]
			}
			if $lagging {
				drawsingle $object $drawing $lname $lx $ly
				set lagging 0
			}
		}
	}
	if $lagging {
		drawsingle $object $drawing $lname $lx $ly
	}
}

