# Procedures used by the bindings
# -------------------------------

# Basic drawing tools
# -------------------
proc drawconnect {canvas line item1 item2 {fraction 1}} {
#putsvars canvas line item1 item2 fraction
	foreach item [list $item1 $item2] num {1 2} {
		foreach {bx1 by1 bx2 by2} [$canvas bbox $item] break
		set size [expr {$by2 - $by1}]
		set temp [expr {$bx2 - $bx1}]
		if {$temp > $size} {
			set size$num [expr {$fraction*$temp/2.0}]
		} else {
			set size$num [expr {$fraction*$size/2.0}]
		}
	}
	foreach {x1 y1} [$canvas coords $item1] break
	foreach {x2 y2} [$canvas coords $item2] break
	set coords [$canvas coords $line]
	set len [llength $coords]
	if {$len == 4} {
		foreach {xd1 yd1 xd2 yd2} [list $x2 $y2 $x1 $y1] break
	} else {
		foreach {temp temp xd1 yd1} $coords break
		foreach {xd2 yd2 temp temp} [lrange $coords [expr {$len-4}] end] break
	}
	foreach {num} {1 2} size [list $size1 $size2] \
		{x y xd yd} [list $x1 $y1 $xd1 $yd1 $x2 $y2 $xd2 $yd2] {
		if {$x == $xd} {
			if {$y == $yd} {
			} elseif {$y < $yd} {
				set y [expr {$y+$size}]
			} else {
				set y [expr {$y-$size}]
			}
		} elseif {$y == $yd} {
			if {$x < $xd} {
				set x [expr {$x+$size}]
			} else {
				set x [expr {$x-$size1}]
			}
		} else {
			if {$x < $xd} {
				set a [expr {3.14159265359+atan(double($yd - $y) / ($xd - $x))}]
			} else {
				set a [expr {atan(double($yd - $y) / ($xd - $x))}]
			}
			set x [expr {$x-cos($a)*$size}]
			set y [expr {$y-sin($a)*$size}]
		}
		set x$num $x
		set y$num $y
	}
	set coords [lreplace $coords 0 1 $x1 $y1]
	set coords [lreplace $coords [expr {$len-2}] [expr {$len-1}] $x2 $y2]
	eval $canvas coords $line $coords
}

proc baseconnect {object drawing name coords items {list {}} {tag {}}} {
	private $object drawinfo canvas
	if {[eval distance [$canvas coords [lindex $items 0]] [$canvas coords [lindex $items 1]]] > $drawinfo($drawing,zoom_connectdist)} {
		set listids [list]
		set nameids [list]
		set tname [list_concat _ $name]
		if ![info exists drawinfo($drawing,$name)] {
			set tags [list $drawing $tname _background _autoconnect]
			if [llength $tag] {eval lappend tags $tag}
			set previtem [list_shift items]
			if {([llength $items] == 1)&&([llength $coords] >= 4)} {
				set item $items
				set line [eval {$canvas create line} $coords {-tags $tags -width $drawinfo($drawing,connectsize)}]
				drawconnect $canvas $line $previtem $item
				lappend nameids $line
				if [string length $list] {
					lappend listids $line
				}
			} else {
				foreach item $items {
					set line [$canvas create line -1 -1 -1 -1 -tags $tags -width $drawinfo($drawing,connectsize)]
					drawconnect $canvas $line $previtem $item
					set previtem $item
					lappend nameids $line
					if [string length $list] {
						lappend listids $line
					}
				}
			}
			set var [list [privatevar $object drawinfo($drawing,$name)]]
			set lvar [list [privatevar $object drawinfo($drawing,$list)]]
			$canvas undo add "\
				unset $var ; \
				set $lvar \[list_remove \[set $lvar\] $listids\] \
				" "\
				lappend $var $nameids; \
				lappend $lvar $listids"
			set drawinfo($drawing,$name) $nameids
			eval lappend drawinfo($drawing,$list) $listids
		} else {
			set litems $drawinfo($drawing,$name)
			set previtem [list_shift items]
			if {([llength $items] == 1)&&([llength $coords] >= 4)} {
				set item $items
				set line $litems
				eval $canvas coords $line $coords
				drawconnect $canvas $line $previtem $item
			} else {
				foreach item $items line $litems {
					drawconnect $canvas $line $previtem $item
					set previtem $item
				}
			}
		}
	} else {
		if [info exists drawinfo($drawing,$name)] {
			eval $canvas delete $drawinfo($drawing,$name)
			set var [list [privatevar $object drawinfo($drawing,$name)]]
			set lvar [list [privatevar $object drawinfo($drawing,$list)]]
			$canvas undo add "\
				unset $var ; \
				set $lvar \[list_remove \[set $lvar\] $drawinfo($drawing,$name)\]
				" "\
				lappend $lvar $drawinfo($drawing,$name); \
				set $var [list $drawinfo($drawing,$name)]"
			if [string length $list] {
				set drawinfo($drawing,$list) [list_lremove $drawinfo($drawing,$list) $drawinfo($drawing,$name)]
			}
			unset drawinfo($drawing,$name)
		}
	}
}

proc deletebaseconnect {object drawing name {list {}}} {
	private $object drawinfo canvas
	if [info exists drawinfo($drawing,$name)] {
		eval $canvas delete $drawinfo($drawing,$name)
		set var [list [privatevar $object drawinfo($drawing,$name)]]
		set lvar [list [privatevar $object drawinfo($drawing,$list)]]
		$canvas undo add "\
			unset $var ; \
			set $lvar \[list_remove \[set $lvar\] $drawinfo($drawing,$name)\]
			" "\
			lappend $lvar $drawinfo($drawing,$name); \
			set $var [list $drawinfo($drawing,$name)]"
		if [string length $list] {
			set drawinfo($drawing,$list) [list_lremove $drawinfo($drawing,$list) $drawinfo($drawing,$name)]
		}
		unset drawinfo($drawing,$name)
	}
}

# Bindings procedures
# --------------------------------------------------------
proc checkloop {object drawing name part} {
	private $object drawinfo canvas
	if ![regexp ^H $name] return
	set hitems [get drawinfo($drawing,$name,ids) ""]
	set objs [get drawinfo($drawing,$name,objs) ""]
	set items ""
	set poss [list_find -glob $objs "l $part *"]
	set items [list_sub $hitems $poss]
	if [llength $items] {
		set item [lindex $items 0]
		if {[eval distance [$canvas coords $item] [$canvas coords [expr $item-1]]] \
			>[expr 2*$drawinfo($drawing,zoom_connectdist)]} {
			arcsingle $object $drawing $name $part
		}
	}
}

proc drawloop {object drawing name part {x {}} {y {}} {update 0}} {
#putsvars object drawing name part x y
	private $object drawinfo canvas
	if ![regexp ^H $name] return
	if {$part == 1} return
	set hitems [get drawinfo($drawing,$name,ids) ""]
	set objs [get drawinfo($drawing,$name,objs) ""]
	set items ""
	set poss [list_find -glob $objs "l $part *"]
	set items [list_sub $hitems $poss]
	if [llength $items] {
		if ![string length $x] {
			set item [lindex $items [expr int([llength $items]/2)]]
			set x [$canvas coords $item]
			set y [list_pop x]
		}
		set start [lindex $hitems [expr {[lindex $poss 0]-1}]]
		set end [lindex $hitems [expr {[lindex $poss end]+1}]]
		set c1 [lrange [$canvas coords $start] 0 1]
		set c2 [lrange [$canvas coords $end] 0 1]
		set points [lrange [eval circlepoints [llength $items] $c1 $c2 $x $y] 2 end]
		foreach id [get drawinfo($drawing,$name,lids) ""] {
			set tags [$canvas itemcget $id -tags]
			foreach {cname cpart num} [lrange [lindex $tags 1] 2 end] {
				if [string_equal $cname $name] {lappend todo($num) $id}
			}
		}
		foreach item $items {x y} $points {
			set num [expr {$item-$drawinfo($drawing,start)}]
			if [info exists todo($num)] {
				poslinked $object $drawing $item $todo($num) $x $y
			}
			$canvas coords $item $x $y
		}
		if $update {redonumbering $object $drawing $name $part}
		regsub {'$} $name {} helix
		baseconnect $object $drawing [list x $name $part] [list_concat $c1 $points $c2] [list_concat $start $items $end] $helix,lids
	} else {
		if [regexp {'$} $name] {
			set objs [list_reverse $objs]
			set hitems [list_reverse $hitems]
		}
		set poss [list_find -glob $objs "d [expr {$part-1}] *"]
		if ![llength $poss] return
		set start [lindex $hitems [lindex $poss end]]
		set poss [list_find -glob $objs "d $part *"]
		if ![llength $poss] return
		set end [lindex $hitems [lindex $poss 0]]
		set c1 [lrange [$canvas coords $start] 0 1]
		set c2 [lrange [$canvas coords $end] 0 1]
		regsub {'$} $name {} helix
		baseconnect $object $drawing [list x $name $part] [concat $c1 $c2] [list $start $end] $helix,lids
	}
}

proc checksingle {object drawing name} {
	private $object drawinfo canvas
	if ![regexp ^S $name] return
	set items [get drawinfo($drawing,$name,ids) ""]
	if [llength $items] {
		set item [lindex $items 0]
		if {[eval distance [$canvas coords $item] [$canvas coords [expr $item-1]]] \
			>[expr 2*$drawinfo($drawing,zoom_connectdist)]} {
			arcsingle $object $drawing $name
		}
	}
}

proc drawsingle {object drawing name {x {}} {y {}} {update 0}} {
#putsvars object drawing name x y
	private $object drawinfo canvas
	if ![regexp ^S $name] return
	set items [get drawinfo($drawing,$name,ids) ""]
	if [llength $items] {
		set tags [$canvas itemcget [lindex $items 0] -tags]
		if [inlist $tags _noauto] {
			return
		}
		if ![string length $x] {
			set item [lindex $items [expr int([llength $items]/2)]]
			set x [$canvas coords $item]
			set y [list_pop x]
		}
		set start [getitemstart $object $drawing $name]
		set end [getitemend $object $drawing $name]
		set c1 [lrange [$canvas coords $start] 0 1]
		set c2 [lrange [$canvas coords $end] 0 1]
		set points [lrange [eval circlepoints [llength $items] $c1 $c2 $x $y] 2 end]
		catch {unset todo}
		foreach id [get drawinfo($drawing,$name,lids) ""] {
			set tags [$canvas itemcget $id -tags]
			foreach {cname cpart num} [lrange [lindex $tags 1] 2 end] {
				if [string_equal $cname $name] {lappend todo($num) $id}
			}
		}
		foreach item $items {x y} $points {
			set num [expr {$item-$drawinfo($drawing,start)}]
			if [info exists todo($num)] {
				poslinked $object $drawing $item $todo($num) $x $y
			}
			$canvas coords $item $x $y
		}
		if $update {redonumbering $object $drawing $name}
		baseconnect $object $drawing [list x $name] [concat $c1 $points $c2] [concat $start $items $end] $name,lids
	} else {
		regsub ^S $name H helix
		if {"$name" == "S"} {
			return
		} else {
			set start [lindex $drawinfo($drawing,$helix,ids) end]
		}
		set pos [lsearch $drawinfo($drawing,elementlist) $helix]
		incr pos
		set endname [lindex $drawinfo($drawing,elementlist) $pos]
		if [string length $endname] {
			set end [lindex $drawinfo($drawing,$endname,ids) 0]
		} else {
			return
		}
		set c1 [lrange [$canvas coords $start] 0 1]
		set c2 [lrange [$canvas coords $end] 0 1]
		baseconnect $object $drawing [list x $name] [concat $c1 $c2] [list $start $end] $name,lids
	}
}

proc drawhairpin {object drawing name {update 0}} {
#putsvars object drawing name
	private $object drawinfo canvas
	if ![regexp ^S $name] return
	set items [get drawinfo($drawing,$name,ids) ""]
	set num [llength $items]
	if [llength $num] {
		set tags [$canvas itemcget [lindex $items 0] -tags]
		if [inlist $tags _noauto] {
			return
		}
		set start [getitemstart $object $drawing $name]
		set end [getitemend $object $drawing $name]
		set c1 [lrange [$canvas coords $start] 0 1]
		set c2 [lrange [$canvas coords $end] 0 1]
		set dist $drawinfo($drawing,zoom_dist)
		foreach {centerx centery} [eval findcenter $c1 $c2 [expr {($num+1)*$dist}]] {}
		set points [eval centercirclepoints $num $c1 $c2 $centerx $centery]
		catch {unset todo}
		foreach id [get drawinfo($drawing,$name,lids) ""] {
			set tags [$canvas itemcget $id -tags]
			foreach {cname cpart num} [lrange [lindex $tags 1] 2 end] {
				if [string_equal $cname $name] {lappend todo($num) $id}
			}
		}
		foreach item $items {x y} $points {
			set num [expr {$item-$drawinfo($drawing,start)}]
			if [info exists todo($num)] {
				poslinked $object $drawing $item $todo($num) $x $y
			}
			$canvas coords $item $x $y
		}
		if $update {redonumbering $object $drawing $name}
		baseconnect $object $drawing [list x $name] [concat $c1 $points $c2] [concat $start $items $end] $name,lids
	} else {
		regsub ^S $name H helix
		if {"$name" == "S"} {
			return
		} else {
			set start [lindex $drawinfo($drawing,$helix,ids) end]
		}
		set pos [lsearch $drawinfo($drawing,elementlist) $helix]
		incr pos
		set endname [lindex $drawinfo($drawing,elementlist) $pos]
		if [string length $endname] {
			set end [lindex $drawinfo($drawing,$endname,ids) 0]
		} else {
			return
		}
		set c1 [lrange [$canvas coords $start] 0 1]
		set c2 [lrange [$canvas coords $end] 0 1]
		baseconnect $object $drawing [list x $name] [concat $c1 $c2] [list $start $end] $name,lids
	}
}

array set defbulge {
	1,0,xm 1
	1,0,y5 1
	1,0,y3 1
	0,1,xm 1
	0,1,y5 1
	0,1,y3 1
	1,1,xm 2
	1,1,y5 0
	1,1,y3 0
	2,2,xm 2
	2,2,y5 1
	2,2,y3 1
	1,2,xm 2
	1,2,y5 0.5
	1,2,y3 1
	2,1,xm 2
	2,1,y5 1
	2,1,y3 0.5
}

proc drawhelix {object drawing name startx starty angle {update 0}} {
#putsvars object drawing name startx starty angle
	global defloop defbulge
	private $object drawinfo canvas
	regsub {'$} $name {} name
	rmflipped $object $drawing $name
	foreach var {xs ys dist dotlen nsdotlen nsbulge} {
		set $var $drawinfo($drawing,zoom_$var)
	}
	set dotwidth $drawinfo($drawing,dotwidth)
	set nsdotwidth $drawinfo($drawing,nsdotwidth)
	set nsshow $drawinfo($drawing,nsshow)
	set x $startx
	set y $starty
	# Set positions
	set line1 [expr $y]
	set line2 [expr $y+$ys]
	set line2a [expr $line2-($dotlen/2)]
	set line2b [expr $line2a+$dotlen]
	set line3 [expr $line2+$ys]
	set line1ns [expr $line1-$nsbulge]
	set line2nsa [expr $line2-($nsdotlen/2)]
	set line2nsb [expr $line2nsa+$nsdotlen]
	set line3ns [expr $line3+$nsbulge]
	# find items
	set p5objs $drawinfo($drawing,$name,objs)
	set p5ids $drawinfo($drawing,$name,ids)
	set p3objs [list_reverse $drawinfo($drawing,$name',objs)]
	set p3ids [list_reverse $drawinfo($drawing,$name',ids)]
	set dotobjs $drawinfo($drawing,$name,dots)
	set dotids $drawinfo($drawing,$name,dotids)
	set p5pos 0
	set p3pos 0
	set dotpos 0
	set p5len [llength $p5objs]
	while 1 {
		set p5obj [lindex $p5objs $p5pos]
		set p5id [lindex $p5ids $p5pos]
		set p3obj [lindex $p3objs $p3pos]
		set p3id [lindex $p3ids $p3pos]
		set dotid [lindex $dotids $dotpos]
		if {("[lindex [lindex $p5objs $p5pos] 0]" == "l")||("[lindex [lindex $p3objs $p3pos] 0]" == "l")} {
			set x [expr {$x-$xs}]
			set p5items ""
			while {"[lindex [lindex $p5objs $p5pos] 0]" == "l"} {
				lappend p5items [lindex $p5ids $p5pos]
				incr p5pos
			}
			set num5p [llength $p5items]
			set p3items ""
			while {"[lindex [lindex $p3objs $p3pos] 0]" == "l"} {
				lappend p3items [lindex $p3ids $p3pos]
				incr p3pos
			}
			set num3p [llength $p3items]
			# Calculate single stranded area ...
			if [info exists defbulge($num5p,$num3p,xm)] {
				set xm [expr $xs*$defbulge($num5p,$num3p,xm)]
				set y5 [expr $line1-$ys*$defbulge($num5p,$num3p,y5)]
				set y3 [expr $line3+$ys*$defbulge($num5p,$num3p,y3)]
				set x5 [expr $x+$xm/2]
				set x3 $x5
				set xend [expr $x+$xm]
			} else {
				if {$num5p<$num3p} {
					set xm [expr ($num5p+2)*$dist]
				} else {
					set xm [expr ($num3p+2)*$dist]
				}
				if {$xm<$xs} {set xm $xs}
				set r [expr ($num5p+$num3p+4)*$dist/(2*3.14159)]
				set a [expr $xm/(2*$r)]
				set xm [expr 2*$r*sin($a)]
				set xend [expr $x+$xm]
				set xb [expr $x+$xm/2]
				if {$num5p!=0} {
					set x5 $xb
					if {$num5p<$num3p} {
						set y5 [expr $line1-0.9*$r*(1-cos($a))]
					} else {
						set y5 [expr $line1-0.9*$r*(1+cos($a))]
					}
					if {$y5>$line1} {set y5 $line1}
				}
				if {$num3p!=0} {
					set x3 $xb
					if {$num5p>=$num3p} {
						set y3 [expr $line3+0.9*$r*(1-cos($a))]
					} else {
						set y3 [expr $line3+0.9*$r*(1+cos($a))]
					}
					if {$y3<$line3} {set y5 $line3}
				}
			}
			# draw single stranded areas
			if {$num5p == 0} {
				#baseconnect extra_${name}_${start} [list $x $line1 $xend $line1] "$start $end" [list $name $part]
			} else {
				set points [lrange [circlepoints $num5p $x $line1 $xend $line1 $x5 $y5] 2 end]
				foreach item $p5items {cx cy} $points {
					#poslinked $item $cx $cy
					$canvas coords $item $cx $cy
				}
				#baseconnect extra_${name}_${part} "$c1 $points $c2" "$start $items $end" [list $name $part]
				#if $update {redonumbering $object $drawing $name $part}
			}
			if {$num3p == 0} {
				#baseconnect extra_${name}_${start} [list $x $line1 $xend $line1] "$start $end" [list $name $part]
			} else {
				set points [lrange [circlepoints $num3p $x $line3 $xend $line3 $x3 $y3] 2 end]
				foreach item $p3items {cx cy} $points {
					#poslinked $item $cx $cy
					$canvas coords $item $cx $cy
				}
				#baseconnect extra_${name}_${part} "$c1 $points $c2" "$start $items $end" [list $name $part]
				#if $update {redonumbering $object $drawing $name $part}
			}
			set x $xend
		} else {
			# draw double stranded areas
			$canvas coords $p5id $x $line1
			if {"[lindex [lindex [$canvas itemcget $dotid -tags] 1] 1]" == "dot"} {
				$canvas coords $dotid $x $line2a $x $line2b
				$canvas itemconfigure $dotid -width $dotwidth
			} else {
				$canvas coords $dotid $x $line2nsa $x $line2nsb
				$canvas itemconfigure $dotid -width $nsdotwidth
				if !$nsshow {
					$canvas lower $dotid _paper
				}
			}
			$canvas coords $p3id $x $line3
			set x [expr {$x+$xs}]
			incr p5pos
			if {$p5pos >= $p5len} break
			incr p3pos
			incr dotpos
		}
	}
	set namepos [lsearch $drawinfo($drawing,elementlist) $name]
	set id [lindex $drawinfo($drawing,names) $namepos]
	$canvas coords $id [expr {$startx+($x-$startx)/2}] [expr {$line3+2*$ys}]
	$canvas dtag _temp _temp
	$canvas addtag _temp items [concat $p5ids $p3ids $dotids $id]
	$canvas rotate _temp $startx $starty $angle
}

proc getitem {object drawing name} {
	private $object drawinfo
	if [regexp ^S $name] {
		return $drawinfo($drawing,$name,ids)
	} else {
		regsub {'$} $name {} name
		return [concat $drawinfo($drawing,$name,ids) $drawinfo($drawing,$name',ids)]
	}
}

proc getitemstart {object drawing name} {
	private $object drawinfo
	set pos [lsearch $drawinfo($drawing,elementlist) $name]
	if {$pos != 0} {
		incr pos -1
		set pname [lindex $drawinfo($drawing,elementlist) $pos]
		return [lindex $drawinfo($drawing,$pname,ids) end]
	} else {
		return $drawinfo($drawing,start)
	}
}

proc getitemend {object drawing name} {
	private $object drawinfo
	set pos [lsearch $drawinfo($drawing,elementlist) $name]
	incr pos 1
	set nname [lindex $drawinfo($drawing,elementlist) $pos]
	if [string length $nname] {
		return [lindex $drawinfo($drawing,$nname,ids) 0]
	} else {
		return $drawinfo($drawing,end)
	}
}

proc getp5end {object drawing name} {
	private $object drawinfo
	return [lindex $drawinfo($drawing,$name,ids) end]
}

proc getp3end {object drawing name} {
	private $object drawinfo
	return [lindex $drawinfo($drawing,$name',ids) 0]
}

