proc arcsingle {object drawing name {part {}}} {
	private $object drawinfo canvas
	if {"$name"==""} return
	set dist $drawinfo($drawing,zoom_dist)
	if [string length $part] {
		set poss [list_find -glob $drawinfo($drawing,$name,objs) "l $part *"]
		set items [list_sub $drawinfo($drawing,$name,ids) $poss]
		set start [lindex $drawinfo($drawing,$name,ids) [expr {[lindex $poss 0]-1}]]
		set end [lindex $drawinfo($drawing,$name,ids) [expr {[lindex $poss end]+1}]]
		if [flipped $object $drawing $name] {
			set temp $end
			set end $start
			set start $temp
		}
		foreach {x1 y1} [lrange [$canvas coords $start] 0 1] {}
		foreach {x2 y2} [lrange [$canvas coords $end] 0 1] {}
		set num [llength $items]
		set len [expr (($num+1)*$dist)/2]
		set distance [eval distance $x1 $y1 $x2 $y2]
		set xc [expr ($x1+$x2)/2]
		set yc [expr ($y1+$y2)/2]
		set xd [expr ($y2-$y1)/$distance]
		set yd [expr -($x2-$x1)/$distance]
		set r [expr $distance/4]
		set px [expr $xc+$r*$xd]
		set py [expr $yc+$r*$yd]
		drawloop $object $drawing $name $part $px $py
	} else {
		set items $drawinfo($drawing,$name,ids)
		set start [getitemstart $object $drawing $name]
		set end [getitemend $object $drawing $name]
		set flipped 0
		if [flipped $object $drawing [getname $canvas $start]] {
			if [flipped $object $drawing [getname $canvas $end]] {
				set flipped 1
			}
		}
		foreach {x1 y1} [$canvas coords $start] {}
		foreach {x2 y2} [$canvas coords $end] {}
		set num [llength $items]
		set len [expr (($num+1)*$dist)/2]
		set distance [eval distance $x1 $y1 $x2 $y2]
		set xc [expr ($x1+$x2)/2]
		set yc [expr ($y1+$y2)/2]
		set xd [expr ($y2-$y1)/$distance]
		set yd [expr -($x2-$x1)/$distance]
		if $flipped {
			set yd [expr -$yd]
		}
		set r [expr $distance/4]
		set px [expr $xc+$r*$xd]
		set py [expr $yc+$r*$yd]
		drawsingle $object $drawing $name $px $py
	}
	validatelinks $object $drawing $name
	$canvas selection redraw
}

proc setangle {object drawing name} {
	private $object drawinfo canvas current
	set current(angle) ""
	if ![regexp {^H} $name] return
	regsub {'$} $name {} name
	set p5 [lindex $drawinfo($drawing,$name,ids) 0]
	set p3 [lindex $drawinfo($drawing,$name',ids) end]
	foreach {x1 y1} [$canvas coords $p5] {}
	foreach {x2 y2} [$canvas coords $p3] {}
	set current(angle) [expr {round(100*([sqangle $x1 $y1 $x2 $y2]+90))/100.0}]
	if {$current(angle) > 180} {set current(angle) [expr {$current(angle) - 360}]}

}

proc orient {args} {
	set newangle [list_pop args]
	checkargs "selecttree @ angle" "object ?drawing name?" $args
	set object [getobj $object]
	private $object drawinfo canvas current
	if ![info exists drawing] {
		if ![info exists current(drawing)] {error "No structure drawing selected"}
		set drawing $current(drawing)
	}
	if ![info exists name] {set name $current(name)}
	set len [llength $args]
	if ![regexp {^H} $name] return
	regsub {'$} $name {} name
	set p5 [lindex $drawinfo($drawing,$name,ids) 0]
	set p3 [lindex $drawinfo($drawing,$name',ids) end]
	foreach {x1 y1} [$canvas coords $p5] {}
	foreach {x2 y2} [$canvas coords $p3] {}
	set angle [expr [sqangle $x1 $y1 $x2 $y2]+90]
	if [flipped $object $drawing $name] {set angle [expr $angle+180]}
	if {$len == 1} {
		$canvas rotate _sel $x1 $y1 [expr {$newangle-$angle}]
		validate $object $drawing
		$canvas selection redraw
	} else {
		$canvas dtag _temp _temp
		set items [list_concat $drawinfo($drawing,$name,ids) $drawinfo($drawing,$name',ids) $drawinfo($drawing,$name,dotids) [get drawinfo($drawing,$name,lids) ""]]
		$canvas addtag _temp items $items
		$canvas rotate _temp $x1 $y1 [expr {$newangle-$angle}]
	}
	set current(angle) $newangle
}

proc placehelix {object drawing name x y angle} {
	private $object drawinfo canvas
	orient $object $drawing $name $angle
	set p [lindex $drawinfo($drawing,$name,ids) 0]
 	foreach {cx cy} [$canvas coords $p] {}
	$canvas move _temp [expr {$x - $cx}] [expr {$y - $cy}]
}

proc arcbase {args} {
	checkargs "arcbase" "object ?drawing? ?name? ?part?" $args
	set object [getobj $object]
	private $object drawinfo canvas current
	if ![info exists drawing] {
		if ![info exists current(drawing)] {error "No structure drawing selected"}
		set drawing $current(drawing)
	}
	if ![info exists name] {set name $current(name)}
	if ![info exists part] {set part $current(part)}
	if [regexp ^H $name] {
		if {$current(type) == "l"} {
			eval arcsingle $object $drawing $name $part
		} else {
			set elementlist $drawinfo($drawing,elementlist)
			regsub {'$} $name {} name
			set pos [list_find -exact $elementlist $name]
			arcsingle $object $drawing [lindex $elementlist [expr {$pos-1}]]
			set pos [list_find -exact $elementlist $name']
			arcsingle $object $drawing [lindex $elementlist [expr {$pos+1}]]
		}
	} elseif {"$name" == "start"} {
		set elementlist $drawinfo($drawing,elementlist)
		arcsingle $object $drawing [lindex $elementlist 0]
	} elseif {"$name" == "end"} {
		set elementlist $drawinfo($drawing,elementlist)
		arcsingle $object $drawing [lindex $elementlist end]
	} else {
		arcsingle $object $drawing $name
	}
}

proc straightensingle {args} {
	checkargs "straightensingle" "object ?drawing? ?name? ?part?" $args
	set object [getobj $object]
	private $object drawinfo canvas current
	if ![info exists drawing] {
		if ![info exists current(drawing)] {error "No structure drawing selected"}
		set drawing $current(drawing)
	}
	if ![info exists name] {set name $current(name)}
	if ![info exists part] {set part $current(part)}
	if {"$name"==""} return
	if [string length $part] {
		foreach {x1 y1} [$canvas coords [getloopstart $object $drawing $name $part]] {break}
		foreach {x2 y2} [$canvas coords [getloopend $object $drawing $name $part]] {break}
		set y [expr ($y1+$y2)/2]
		set x [expr ($x1+$x2)/2]
		drawloop $object $drawing $name $part $x $y
	} elseif ![regexp ^H $name] {
		foreach {x1 y1} [$canvas coords [getitemstart $object $drawing $name]] {break}
		foreach {x2 y2} [$canvas coords [getitemend $object $drawing $name]] {break}
		set y [expr ($y1+$y2)/2]
		set x [expr ($x1+$x2)/2]
		drawsingle $object $drawing $name $x $y
	}
	validatelinks $object $drawing $name
	Classy::todo $canvas selection redraw
}

proc straightenbase {args} {
	checkargs "straightenbase" "object ?drawing? ?name? ?part?" $args
	set object [getobj $object]
	private $object drawinfo canvas current
	if ![info exists drawing] {
		if ![info exists current(drawing)] {error "No structure drawing selected"}
		set drawing $current(drawing)
	}
	if ![info exists name] {set name $current(name)}
	if ![info exists part] {
		if [string_equal $current(type) l] {
			set part $current(part)
		} else {
			set part ""
		}
	}
	if {"$name"==""} return
	if [string length $part] {
		straightensingle $object $drawing $name $part
	} elseif [regexp ^H $name] {
		regsub {'$} $name {} name
		set elementlist $drawinfo($drawing,elementlist)
		set pos [lsearch -exact $elementlist $name]
		straightensingle $object $drawing [lindex $elementlist [expr {$pos-1}]] {}
		set pos [lsearch -exact $elementlist $name']
		straightensingle $object $drawing [lindex $elementlist [expr {$pos+1}]] {}
	} elseif {"$name"=="start"} {
		straightensingle $object $drawing [lindex $drawinfo($drawing,elementlist) 0] {}
	} elseif {"$name"=="end"} {
		straightensingle $object $drawing [lindex $drawinfo($drawing,elementlist) end] {}
	} else {
		straightensingle $object $drawing $name
	}
}

proc straightentop {args} {
	checkargs "straightenbase" "object ?drawing? ?name? ?part?" $args
	set object [getobj $object]
	private $object drawinfo canvas current
	if ![info exists drawing] {
		if ![info exists current(drawing)] {error "No structure drawing selected"}
		set drawing $current(drawing)
	}
	if ![info exists name] {set name $current(name)}
	if ![info exists part] {
		if [string_equal $current(type) l] {
			set part $current(part)
		} else {
			set part ""
		}
	}
	if {"$name"==""} return
	if [string length $part] {
		straightensingle $object $drawing $name $part
	} elseif [regexp ^H $name] {
		regsub {'$} $name {} name
		set elementlist $drawinfo($drawing,elementlist)
		set pos [lsearch -exact $elementlist $name]
		straightensingle $object $drawing [lindex $elementlist [expr {$pos+1}]] {}
		set pos [lsearch -exact $elementlist $name']
		straightensingle $object $drawing [lindex $elementlist [expr {$pos-1}]] {}
	} elseif {"$name"=="start"} {
		straightensingle $object $drawing [lindex $drawinfo($drawing,elementlist) 0] {}
	} elseif {"$name"=="end"} {
		straightensingle $object $drawing [lindex $drawinfo($drawing,elementlist) end] {}
	} else {
		straightensingle $object $drawing $name
	}
}

proc redrawhelix {args} {
	checkargs "redrawhelix" "object ?drawing name?" $args
	set object [getobj $object]
	private $object drawinfo canvas current
	if ![info exists drawing] {
		if ![info exists current(drawing)] {error "No structure drawing selected"}
		set drawing $current(drawing)
	}
	if ![info exists name] {set name $current(name)}
	if ![regexp {^H} $name] continue
	regsub {'$} $name {} name
	set items [list_concat $drawinfo($drawing,$name,ids) $drawinfo($drawing,$name',ids)]
	foreach item $items tags [$canvas mitemcget $items -tags] {
		set type [lindex [lindex $tags 1] 1]
		if [string_equal $type c] {
			$canvas itemconfigure $item -tags [lreplace $tags 1 1 [lreplace [lindex $tags 1] 1 1 d]]
		}
	}
	foreach item $drawinfo($drawing,$name,dotids) {
		$canvas dtag $item _hidden
		$canvas raise $item $drawinfo($drawing,start)
	}
	set tomove {}
	foreach item [list_concat [get drawinfo($drawing,$name,lids) ""] [get drawinfo($drawing,$name',lids) ""]] {
		set tags [$canvas itemcget $item -tags]
		if ![llength $tags] continue
		set obj [lindex $tags 1]
		switch [lindex $obj 1] {
			box {
				if [inlist $tags _connected] {deletebox $object $item}
			}
			bc {
				if [inlist $tags _connected] {deleteconnectbases $object $item}
			}
			x {
				set cnames([lrange $obj 1 end]) 1
			}
			lnk {
				set lo [expr {[lindex $obj 4]+$drawinfo($drawing,start)}]
				if [isint $lo] {
					set diffcoords ""
					foreach {rx ry} [$canvas coords $lo] break
					foreach {x y} [$canvas coords $item] {
						lappend diffcoords [expr {$x-$rx}] [expr {$y-$ry}]
					}
					lappend tomove $item $lo $diffcoords
				}
			}
		}
	}
	foreach cname [array names cnames] {
		deletebaseconnect $object $drawing $cname
	}
	if [flipped $object $drawing $name] {
		set flipped 1
		fliphelix $object $drawing $name
	}
	set p5 [lindex $drawinfo($drawing,$name,ids) 0]
	set p3 [lindex $drawinfo($drawing,$name',ids) end]
	foreach {x1 y1} [$canvas coords $p5] {}
	foreach {x2 y2} [$canvas coords $p3] {}
	set angle [expr [sqangle $x1 $y1 $x2 $y2]+90]
	drawhelix $object $drawing $name $x1 $y1 $angle 1
	if [info exists flipped] {
		fliphelix $object $drawing $name
	}
	if [llength $tomove] {
		foreach {item lo diffcoords} $tomove {
			foreach {rx ry} [$canvas coords $lo] break
			set newcoords ""
			foreach {x y} $diffcoords {
				lappend newcoords [expr {$rx+$x}] [expr {$ry+$y}]
			}
			eval $canvas coords $item $newcoords
		}
	}
	validate $object $drawing
	if {"$drawinfo($drawing,nxt,$name)" == "$drawinfo($drawing,prv,$name')"} {
		drawhairpin $object $drawing $drawinfo($drawing,nxt,$name) 1
	} else {
		drawsingle $object $drawing $drawinfo($drawing,nxt,$name) {} {} 1
		drawsingle $object $drawing $drawinfo($drawing,prv,$name') {} {} 1
	}
	drawsingle $object $drawing $drawinfo($drawing,prv,$name)
	drawsingle $object $drawing $drawinfo($drawing,nxt,$name')
	$canvas selection redraw
}

proc redrawhelices {args} {
	checkargs "redrawhelix" "object ?drawing name?" $args
	set object [getobj $object]
	private $object drawinfo canvas current
	if ![info exists drawing] {
		if ![info exists current(drawing)] {error "No structure drawing selected"}
		set drawing $current(drawing)
	}
	set groups [list_remdup [list_subindex [list_subindex [$canvas mitemcget _sel -tags] 1] 2]]
	foreach name $groups {
		if [regexp {^H.*[^']$} $name] {
			redrawhelix $object $drawing $name
		}
	}
}

proc centerdrawing {args} {
	checkargs "centerdrawing" "object ?drawing?" $args
	set object [getobj $object]
	private $object drawinfo canvas current
	if ![info exists drawing] {
		if ![info exists current(drawing)] {error "No structure drawing selected"}
		set drawing $current(drawing)
	}
	set box [$canvas bbox $drawing]
	set x1 [lindex $box 0]
	set y1 [lindex $box 1]
	set x2 [lindex $box 2]
	set y2 [lindex $box 3]
	set paper [$canvas bbox _paper]
	set movex [expr [lindex $paper 2]/2-($x2-$x1)/2-$x1]
	set movey [expr [lindex $paper 3]/2-($y2-$y1)/2-$y1]
	$canvas move $drawing $movex $movey
	$canvas selection redraw
}

proc angleentry {w} {
	Classy::NumEntry $w -label Angle
	return [varsubst w {
		$w configure -command [list orient %W] -textvariable [privatevar %W current(angle)]
	}]
}

proc getloopstart {object drawing name {part {}}} {
	private $object drawinfo
	if {"$name"==""} return
	if [string length $part] {
		set poss [list_find -glob $drawinfo($drawing,$name,objs) "l $part *"]
		set items [list_sub $drawinfo($drawing,$name,ids) $poss]
		return [lindex $drawinfo($drawing,$name,ids) [expr {[lindex $poss 0]-1}]]
	} else {
		set items $drawinfo($drawing,$name,ids)
		return [getitemstart $object $drawing $name]
	}
}

proc getloopend {object drawing name {part {}}} {
	private $object drawinfo
	if {"$name"==""} return
	if [string length $part] {
		set poss [list_find -glob $drawinfo($drawing,$name,objs) "l $part *"]
		set items [list_sub $drawinfo($drawing,$name,ids) $poss]
		return [lindex $drawinfo($drawing,$name,ids) [expr {[lindex $poss end]+1}]]
	} else {
		set items $drawinfo($drawing,$name,ids)
		return [getitemend $object $drawing $name]
	}
}


