bind LinkMode <<Action>> {linkaction %W %x %y}

bind LinkMode <Escape> {
	set object [getobj %W]
	private $object link canvas
	focus $canvas
	if [info exists link] {
		bindtags $canvas $link(bindtags)
		catch {unset link}
		nomsg $object
	}
}

proc linkaction {w x y} {
	set object [getobj $w]
	private $object drawinfo current canvas link
	focus $canvas
	set x [$canvas canvasx $x]
	set y [$canvas canvasy $y]
	set list [$canvas find overlapping [expr {$x-1}] [expr {$y-1}] [expr {$x+1}] [expr {$y+1}]]
	set item [lindex $list end]
	if [llength $item] {
		bindtags $canvas $link(bindtags)
		foreach curitem $link(items) {
			addlink $object $item $curitem
		}
		catch {unset link}
		nomsg $object
	} else {
		error "Nothing selected (Press Escape to cancel link)"
	}
}

proc addlink {object bases items {type lnk}} {
	private $object canvas drawinfo
	set id [list _ $type]
	set bnames ""
	foreach base $bases btags [$canvas mitemcget $bases -tags] {
		if {[lsearch $btags _base]==-1} {
			error "Objects can only be linked to bases"
		}	
		set bdrawing [lindex $btags 0]
		set btag [lindex $btags 1]
		lappend id [lindex $btag 2] [lindex $btag 3] [expr {$base - $drawinfo($bdrawing,start)}]
		lappend bnames [lindex $btag 2]
	}
	foreach item $items {
		unlink $object $item
		set otags [$canvas itemcget $item -tags]
		set odrawing [lindex $otags 0]
		if {[lsearch $drawinfo(drawings) $odrawing] != -1} {error "This object cannot be linked because it already belongs to a structure drawing"}
		set otags [list_concat [list $bdrawing $id] $otags]
		$canvas itemconfigure $item -tags $otags
		foreach bname $bnames {
			list_addnew drawinfo($bdrawing,$bname,lids) $item
			set var [list [privatevar $object drawinfo($bdrawing,$bname,lids)]]
			$canvas undo add \
				"list_addnew $var $item" \
				"set $var \[list_remove \[set $var\] $item\]"
		}
	}
}

proc unlink {object tag} {
	set object [getobj $object]
	private $object canvas drawinfo
	if ![isint [lindex $tag 0]] {
		set tag [$canvas find withtag $tag]
	}
	foreach item $tag {
		set otags [$canvas itemcget $item -tags]
		set odrawing [lindex $otags 0]
		if {[lsearch $drawinfo(drawings) $odrawing] == -1} {return}
		set ot [lindex $otags 1]
		set otype [lindex $ot 1]
		if ![inlist {lnk num numl name bc box} $otype] {return}
		foreach {oname opart opos} [lrange $ot 2 end] {
			set drawinfo($odrawing,$oname,lids) [list_remove $drawinfo($odrawing,$oname,lids) $item]
			set var [list [privatevar $object drawinfo($odrawing,$oname,lids)]]
			$canvas undo add \
				"set $var \[list_remove \[set $var\] $item\]" \
				"list_addnew $var $item"
		}
		$canvas itemconfigure $item -tags [lrange $otags 2 end]
	}
}

proc findlinked {object item} {
	private $object drawinfo canvas
	set tags [$canvas itemcget $item -tags]
	set drawing [lindex $tags 0]
	set itags [lindex $tags 1]
	set name [lindex $itags 2]
	set num [expr {$item-$drawinfo($drawing,start)}]
	set result ""
	foreach id [get drawinfo($drawing,$name,lids) ""] {
		set tags [$canvas itemcget $id -tags]
		foreach cnum [lrange [lindex $tags 1] 4 end] {
			if {$cnum == $num} {
				lappend result $id
			}
		}
	}
	return $result
}

proc poslinked {object drawing item items x y} {
	private $object canvas
	set sx [$canvas coords $item]
	set sy [list_pop sx]
	set mx [expr {$x-$sx}]
	set my [expr {$y-$sy}]
	foreach id $items {
		$canvas move $id $mx $my
	}
}

proc link {w} {
	set object [getobj $w]
	private $object drawinfo current canvas link
	set link(items) [$canvas selection get]
	foreach item $link(items) {
		set itags [$canvas itemcget $item -tags]
		set group [lindex $itags 0]
		if [inlist $drawinfo(drawings) $group] {
			error "One of the selected objects already belongs to a structure drawing, so linking is impossible."
		}
	}
	set link(bindtags) [bindtags $canvas]
	bindtags $canvas LinkMode
	msg $object "Click on base to link to"
}
