proc loadct {object file {skeletonfile {}}} {
	if {"$file"==""} return
	private $object loadct loadali canvas current
	set undostatus [$canvas undo status]
	$canvas undo 0
	if [info exists loadct] {unset loadct}
	set f [open $file]
	set c [split [read $f] "\n"]
	close $f
	set pos 0
	set loadct(list) [list]
	set loadct(data) [list]
	while 1 {
		set line [lindex $c $pos]
		if ![regexp {([0-9]+) (.*)} $line temp size descr] {
			break
		}
		lappend loadct(list) $descr
		lappend loadct(data) [lrange $c $pos [expr {$pos+$size}]]
		incr pos $size
		incr pos
	}
	if {[llength $loadct(list)]>1} {
		Classy::Dialog .loadct -title "Multiple entries" -closecommand {set loadct(stop) 1}
		.loadct add go "Go" {destroy .loadct} default
		Classy::OptionBox .loadct.options.opt -label "File contains multiple entries" \
			-variable loadct(opt) -orient vertical
		.loadct.options.opt add batch "Batch job: one structure per file" -command {
			set loadct(file) [Classy::savefile -title "Save files as" -help multiple_saveas \
				-filetypes {{"RnaViz structure" .str} {"All files" *}} -defaultextension .str]
		}
		.loadct.options.opt add load "Load selected structures on current page"
		pack .loadct.options.opt -fill x
		set loadct(opt) load
		tkwait window .loadct
		if [info exists loadct(stop)] {
			unset loadct
			return
		}
		Classy::SelectDialog .loadctselect -title "Select structures" -selectmode persistent \
			-command "invoke args \{set [privatevar $object loadct(select)] \[.loadctselect curselection\]\}"
		.loadctselect fill $loadct(list)
		tkwait window .loadctselect
		set loadct(list) [list_sub $loadct(list) $loadct(select)]
		set loadct(data) [list_sub $loadct(data) $loadct(select)]
	}
	set loadct(opt) load
	Classy::busy $object
	foreach name $loadct(list) data $loadct(data) {
		busy $object "Loading ct entry" [llength $data]
		busyincr $object
		ct_paste $object $data $skeletonfile
		if {"$loadct(opt)"=="batch"} {
			savestruct [file root $loadct(file)]$listnum[file extension $loadct(file)]
		}
	}
	Classy::busy remove
	$canvas undo $undostatus
}

proc ct_paste {object ctdata skeletonfile} {
#	uplevel #0 [list set object $object]
#	uplevel #0 [list set ctdata $ctdata]
#	uplevel #0 [list set skeletonfile $skeletonfile]
	set header [list_shift ctdata]
	regexp "^\[ \t\]+(\[0-9\]+)\[ \t\]+(.*)$" $header temp num rnaname
	# read data
	set sequence ""
	foreach line $ctdata {
		foreach {pos cbase temp temp comp} $line {break}
		lappend sequence $cbase
		if {$comp == 0} continue
		set bp($pos) $comp
		set bp($comp) $pos
	}
	set len [llength $sequence]
	catch {unset names}
	catch {unset kname}
	catch {unset ktype}
	catch {unset kside}
	catch {unset kpart}
	catch {unset ns}
	set hnum 0
	foreach pos1 [lsort -integer [array names bp]] {
		if [info exists kname($pos1)] continue
		set pos2 $bp($pos1)
		catch {unset name}
		set part d1
		set prev1 $pos1
		while {$prev1 > 0} {
			incr prev1 -1
			if [info exists bp($prev1)] break
			if [info exists kname($prev1)] break
		}
		set prev2 $pos2
		while {$prev2 <= $len} {
			incr prev2 1
			if [info exists bp($prev2)] break
			if [info exists kname($prev2)] break
		}
		if {[info exists kname($prev1)]&&[info exists kname($prev2)]&&[string equal $kname($prev1) $kname($prev2)]} {
			set name $kname($prev1)
			set part $kpart($prev1)
			set pnum [string range $part 1 end]
			set loop 0
			set pos $prev1
			incr pos
			while {$pos < $pos1} {
				set loop 1
				set ktype($pos) l
				set kname($pos) $name
				set kside($pos) p5
				set kpart($pos) s$pnum
				incr pos
			}
			set pos $prev2
			incr pos -1
			while {$pos > $pos2} {
				set loop 1
				set ktype($pos) l
				set kname($pos) $name
				set kside($pos) p3
				set kpart($pos) s$pnum'
				incr pos -1
			}
			if $loop {
				incr pnum
				set part d$pnum
			}
		}
		if ![info exists name] {
			while 1 {
				incr hnum
				set name H$hnum
				if ![info exists names($name)] break
			}
			set names($name) 1
		}
		set ktype($pos1) d
		set ktype($pos2) d
		set kname($pos1) $name
		set kname($pos2) $name
		set kside($pos1) p5
		set kside($pos2) p3
		set kpart($pos1) $part
		set kpart($pos2) $part
	}
	set snum 0
	set hnum 1
	set scur 1
	set pos 1
	set data {}
	foreach base $sequence {
		set type [get ktype($pos) s]
		if [info exists kname($pos)] {
			set name $kname($pos)
		} else {
			if [string equal $type s] {
				if !$scur {
					incr snum
					set scur 1
				}
				set name S$snum
			} else {
				if $scur {
					incr hnum
					set scur 0
				}
				set name H$hnum
			}
		}
		set el [list $base $pos - $name $type]
		if ![string equal $type s] {
			set scur 0
			lappend el [get kpart($pos) d1] $kside($pos) [get ns($pos) s]
		}
		lappend data $el
		incr pos
	}
	createstruct $object $rnaname $data $skeletonfile
}

