proc rnaml_load {object file {skeletonfile {}}} {
	if {"$file"==""} return
	private $object loadali canvas current
	set undostatus [$canvas undo status]
	$canvas undo 0
	set map [map_xml [file_read $file]]
	set ids [list_select [map_fields $map rnaml] rna*]
	if {[llength $ids] > 1} {
		set select ""
		foreach rna $ids {
			lappend select [list $rna [map_get $map [list rnaml $rna _args name]]]
		}
		Classy::SelectDialog .loadaliselect -title "Select structures" \
		-help multiple_select -selectmode multiple -command [list set [privatevar $object loadali(list)]]
		.loadaliselect fill $select
		tkwait window .loadaliselect
		set ids [list_subindex $loadali(list) 0]
	}
	if {[llength $ids]>1} {
		Classy::Dialog .loadali -title "Multiple entries" -closecommand {set loadali(stop) 1} -help multiple_entries
		.loadali add go "Go" {destroy .loadali} default
		Classy::OptionBox .loadali.options.opt -label "File contains multiple entries" \
			-variable loadali(opt) -orient vertical
		.loadali.options.opt add batch "Batch job: one structure per file" -command {
			set loadali(file) [Classy::savefile -title "Save files as" -help multiple_saveas]
		}
		.loadali.options.opt add load "Load selected structures on current page"
		pack .loadali.options.opt -fill x
		set loadali(opt) load
		tkwait window .loadali
		if [info exists loadali(stop)] {
			unset loadali
			return
		}
	} else {
		set loadali(opt) load
	}
	Classy::busy $object
	foreach id $ids {
		if {"$loadali(opt)"=="batch"} {
			clearpage
		}
		rnaml_paste $object [map_get $map [list rnaml $id]] $skeletonfile
		if {"$loadali(opt)"=="batch"} {
			savestruct [file root $loadali(file)]$id[file extension $loadali(file)]
		}
	}
	Classy::busy remove
	$canvas undo $undostatus
}

proc rnaml_paste {object map {skeletonfile {}}} {
#	uplevel #0 [list set object $object]
#	uplevel #0 [list set map $map]
#	uplevel #0 [list set skeletonfile $skeletonfile]
	set sequence [map_get $map {sequence data _data}]
	regsub -all {[^A-Za-z]} $sequence {} sequence
	set len [string length $sequence]
	set sequence [split $sequence {}]
	set rnaname [map_get $map {_args name}]
	set ss [map_get $map structure]
	catch {set ssdisplay [map_get $ss secondary-struct-display]}
	set ssfields [map_fields $ss]
	set helices [list_select $ssfields helix*]
	set singles [list_select $ssfields single-strand*]
	set basepairs [list_select $ssfields base-pair*]
	set data ""
	set pos 0
	catch {unset names}
	catch {unset kname}
	catch {unset ktype}
	catch {unset kside}
	catch {unset kpart}
	catch {unset ns}
	foreach single $singles {
		set args [map_get $ss [list $single _args]]
		set name S[map_get $args id]
		set pos [map_get $args start-base-num]
		if [catch {map_get $args length} len] {
			set to [map_get $args stop-base-num]
		} else {
			set to [expr {$pos+$len}]
		}
		set names($name) 1
		for {} {$pos <= $to} {incr pos} {
			set ktype($pos) s
			set kname($pos) $name
		}
	}
	foreach helix $helices {
		set args [map_get $ss [list $helix _args]]
		set name H[map_get $args id]
		set len [map_get $args length]
		set pos1 [map_get $args _5p-base-num]
		set pos2 [map_get $args _3p-base-num]
		set names($name) 1
		for {set i 0} {$i < $len} {incr i} {
			set ktype($pos1) d
			set ktype($pos2) d
			set kname($pos1) $name
			set kname($pos2) $name
			set kside($pos1) p5
			set kside($pos2) p3
			set kpart($pos1) d1
			set kpart($pos2) d1
			incr pos1
			incr pos2 -1
		}
	}
	set hnum 0
	catch {unset bp}
	foreach basepair $basepairs {
		set args [map_get $ss [list $basepair _args]]
		set pos1 [map_get $args _5p-base-num]
		set pos2 [map_get $args _3p-base-num]
		set bp($pos1) $pos2
		set bp($pos2) $pos1
	}
	foreach pos1 [lsort -integer [array names bp]] {
		if [info exists kname($pos1)] continue
		set pos2 $bp($pos1)
		catch {unset name}
		set part d1
		set prev1 $pos1
		while {$prev1 > 0} {
			incr prev1 -1
			if [info exists bp($prev1)] break
			if [info exists kname($prev1)] break
		}
		set prev2 $pos2
		while {$prev2 <= $len} {
			incr prev2 1
			if [info exists bp($prev2)] break
			if [info exists kname($prev2)] break
		}
		if {[info exists kname($prev1)]&&[info exists kname($prev2)]&&[string equal $kname($prev1) $kname($prev2)]} {
			set name $kname($prev1)
			set part $kpart($prev1)
			set pnum [string range $part 1 end]
			set loop 0
			set pos $prev1
			incr pos
			while {$pos < $pos1} {
				set loop 1
				set ktype($pos) l
				set kname($pos) $name
				set kside($pos) p5
				set kpart($pos) s$pnum
				incr pos
			}
			set pos $prev2
			incr pos -1
			while {$pos > $pos2} {
				set loop 1
				set ktype($pos) l
				set kname($pos) $name
				set kside($pos) p3
				set kpart($pos) s$pnum'
				incr pos -1
			}
			if $loop {
				incr pnum
				set part d$pnum
			}
		}
		if ![info exists name] {
			while 1 {
				incr hnum
				set name H$hnum
				if ![info exists names($name)] break
			}
			set names($name) 1
		}
		set ktype($pos1) d
		set ktype($pos2) d
		set kname($pos1) $name
		set kname($pos2) $name
		set kside($pos1) p5
		set kside($pos2) p3
		set kpart($pos1) $part
		set kpart($pos2) $part
	}
	set snum 0
	set hnum 1
	set scur 1
	set pos 1
	set data {}
	foreach base $sequence {
		set type [get ktype($pos) s]
		if [info exists kname($pos)] {
			set name $kname($pos)
		} else {
			if [string equal $type s] {
				if !$scur {
					incr snum
					set scur 1
				}
				set name S$snum
			} else {
				if $scur {
					incr hnum
					set scur 0
				}
				set name H$hnum
			}
		}
		set el [list $base $pos - $name $type]
		if ![string equal $type s] {
			set scur 0
			lappend el [get kpart($pos) d1] $kside($pos) [get ns($pos) s]
		}
		lappend data $el
		incr pos
	}
	createstruct $object $rnaname $data $skeletonfile
}
