proc saveskeleton {args} {
	checkargs saveskeleton "object ?drawing? ?file?" $args
	set object [getobj $object]
	private $object drawinfo canvas current
	if ![info exists drawing] {
		if ![info exists current(drawing)] {error "No structure drawing selected"}
		set drawing $current(drawing)
	}
	if ![inlist $drawinfo(drawings) $drawing] {
		error "Select a structure drawing first"
	}
	if ![info exists file] {
		set file [Classy::savefile -title "Save skeleton" -filter *.skl -help save_skeleton -filetypes {{"Skeleton file" .skl} {"All files" *}} -defaultextension .skl]
	}
	if ![string length $file] return
	if ![Classy::overwriteyn $file noappend] return
	set elementlist $drawinfo($drawing,elementlist)
	update idletasks
	if ![inlist $drawinfo(drawings) $drawing] {
		error "\"$drawing\" is not a structure drawing"
	}
	busy $object "Saving skeleton to file \"$file\"" [llength $elementlist]
	Classy::busy add $object
	set zoom [$canvas zoom]
	$canvas zoom 1
	set f [open $file w]
	puts $f "#RnaViz skeleton 2.0"
	set options ""
	lappend options [getsettings $object $drawing]
	if [flipped $object $drawing]	{
		lappend options "flipped"
		flipdrawing $object $drawing
	}
	puts $f "options $options"
	puts $f "start [$canvas coords $drawinfo($drawing,start)]"
	puts $f "end [$canvas coords $drawinfo($drawing,end)]"
	foreach item [list_sub $elementlist [list_find -regexp $elementlist {^H.*[^']$}]] {
		if [flipped $object $drawing $item] {
			set flip 1
			fliphelix $object $drawing $item
		}
		set p5 [lindex $drawinfo($drawing,$item,ids) 0]
		set p3 [lindex $drawinfo($drawing,$item',ids) end]
		foreach {x1 y1} [$canvas coords $p5] {}
		foreach {x2 y2} [$canvas coords $p3] {}
		set angle [expr {[sqangle $x1 $y1 $x2 $y2]+90}]
		if [info exists flip] {
			unset flip
			puts $f "$item $x1 $y1 $angle f"
			fliphelix $object $drawing $item
		} else {
			puts $f "$item $x1 $y1 $angle"
		}
		busyincr $object
	}
	catch {unset done}
	foreach item [list_sub $elementlist [list_find -regexp $elementlist ^S]] {
		set items [get drawinfo($drawing,$item,ids)]
		set num [llength $items]
		if $num {
			set pos [expr {($num-1)/2}]
			puts $f "$item [$canvas coords [lindex $items $pos]]"
		}
		set done($item) 1
		busyincr $object
	}
	foreach item [lsort -int [$canvas find andtags $drawing _helixname]] {
		set name [$canvas itemcget $item -text]
		puts $f "n$name [$canvas coords $item]"
	}
	close $f
	if {[lsearch $options flipped]!=-1}	{
		flipall $drawing
	}
	$canvas zoom $zoom
	Classy::busy remove $object
	notbusy $object
}

proc loadskeleton {args} {
	checkargs saveskeleton "object ?drawing? ?file?" [lrange $args 0 2]
	set args [lrange $args 3 end]
	set object [getobj $object]
	private $object drawinfo canvas current
	if ![info exists drawing] {
		if ![info exists current(drawing)] {error "No structure drawing selected"}
		set drawing $current(drawing)
	}
	if ![inlist $drawinfo(drawings) $drawing] {
		error "Select a structure drawing first"
	}
	if ![info exists file] {
		set file [Classy::selectfile -title "Select skeleton" -filter *.skl -help save_skeleton -filetypes {{"Skeleton file" .skl} {"All files" *}} -defaultextension .skl]
	}
	set elementlist $drawinfo($drawing,elementlist)
	set redraw [get current(skeletonredraw) 1]
	set skeletonfonts [get current(skeletonfonts) 1]
	set batch 0
	foreach {option value} $args {
		if ![string length $value] {
			error "error in options"
		}
		switch -- $option {
			-redrawhelices {
				set redraw [true $value]
			}
			-skeletonfonts {
				set skeletonfonts [true $value]
			}
			-batch {
				set batc [true $value]
			}
		}
	}
	if $redraw {
		set pattern $drawing,*,x*
		set helixcmd drawhelix
	} else {
		set pattern $drawing,*,x
		set helixcmd  placehelix
	}
	if ![string length $file] return
#	removeconnected $drawing c
	if !$batch {
		Classy::busy add $object
		set estep [expr {[llength $elementlist]/90}]
		if $skeletonfonts {
			set size [expr {[llength $elementlist]+10*$estep+1}]
		} else {
			set size [expr {[llength $elementlist]+1}]
		}
		busy $object "Reading skeleton from file \"$file\"" $size
	}
	set f [open $file]
	set line [gets $f]
	set zoom [$canvas zoom]
	switch $line {
		"#RnaViz skeleton" {
			$canvas zoom 10
		}
		"#RnaViz skeleton 2.0" {
			$canvas zoom 1
		}
		default {
			error "\"$file\" is not a valid RnaViz skeleton file"
		}
	}
	catch {unset skel}
	set settingsline [gets $f]
	setsettings $object $drawing [lindex $settingsline 1]
	set skel(options) [lrange $settingsline 2 end]
	set flipped ""
	while 1 {
		set line [gets $f]
		if [eof $f] break
		set name [lindex $line 0]
		set skel($name,x) [lindex $line 1]
		set skel($name,y) [lindex $line 2]
		if [regexp ^H $name] {
			set skel($name,angle) [lindex $line 3]
			if {"[lindex $line 4]"=="f"} {
				set skel($name,flip) 1
			}
		}
	}
	close $f
	if $skeletonfonts {
		if !$batch {busyincr $object $estep}
		$canvas dtag _tempconf _tempconf
		$canvas addtag _tempconf andtags $drawing _base
		$canvas addtag _tempconf withtag $drawinfo($drawing,start)
		$canvas addtag _tempconf withtag $drawinfo($drawing,end)
		if !$batch {busyincr $object $estep}
		$canvas itemconfigure _tempconf -font $drawinfo($drawing,basefont)
		if !$batch {busyincr $object $estep}
		$canvas dtag _tempconf _tempconf
		$canvas addtag _tempconf andtags $drawing _helixname
		if !$batch {busyincr $object $estep}
		$canvas itemconfigure _tempconf -font $drawinfo($drawing,helixfont)
		if !$batch {busyincr $object $estep}
		$canvas dtag _tempconf _tempconf
		$canvas addtag _tempconf andtags $drawing _dot
		if !$batch {busyincr $object $estep}
		$canvas itemconfigure _tempconf -width $drawinfo($drawing,dotwidth)
		if !$batch {busyincr $object $estep}
		$canvas dtag _tempconf _tempconf
		$canvas addtag _tempconf andtags $drawing _nsdot
		if !$batch {busyincr $object $estep}
		$canvas itemconfigure _tempconf -width $drawinfo($drawing,nsdotwidth)
		if !$batch {busyincr $object $estep}
	}
	foreach tag [array names drawinfo $pattern] {
		regexp {,([^,]*),} $tag temp name
		if [regexp ^H $name] {
			regsub {'$} $name {} name
		}
		set drawinfo($drawing,$name,lids) [list_lremove $drawinfo($drawing,$name,lids) $drawinfo($tag)]
		eval $canvas delete $drawinfo($tag)
		unset drawinfo($tag)
	}
	$canvas coords $drawinfo($drawing,start) $skel(start,x) $skel(start,y)
	$canvas coords $drawinfo($drawing,end) $skel(end,x) $skel(end,y)
	unflip $object $drawing
	set dist $drawinfo($drawing,zoom_dist)
	set loop [findloop all $elementlist]
	set size [findloopsize $object $drawing $loop]
	foreach {startx starty} [$canvas coords $drawinfo($drawing,start)] {}
	foreach {endx endy} [$canvas coords $drawinfo($drawing,end)] {}
	foreach {centerx centery} [findcenter $startx $starty $endx $endy [expr {($size+1)*$dist}]] {}
	set points [centercirclepoints $size $startx $starty $endx $endy $centerx $centery]
	drawskeletonitems $object $drawing $loop $points $helixcmd
	if !$batch {busyincr $object}
	foreach item $elementlist {
		if [regexp {^H.*[^']$} $item] {
			set loop [findloop $item $elementlist]
			set size [findloopsize $object $drawing $loop]
			if {[llength $loop]==1} {
				if [info exists skel($loop,x)] {unset skel($loop,x)}
			}
			if [info exists skel($item,flip)] {
				set end [$canvas coords [lindex $drawinfo($drawing,$item,ids) end]]
				set start [$canvas coords [lindex $drawinfo($drawing,$item',ids) 0]]
			} else {
				set start [$canvas coords [lindex $drawinfo($drawing,$item,ids) end]]
				set end [$canvas coords [lindex $drawinfo($drawing,$item',ids) 0]]
			}
			foreach {startx starty} $start {}			
			foreach {endx endy} $end {}			
			set temp [expr {($size+1)*$dist}]
			set temp2 [expr {2.4*[distance $startx $starty $endx $endy]}]
			if {$temp<$temp2} {set temp $temp2}
			set centerx [findcenter $startx $starty $endx $endy $temp]
			set centery [list_pop centerx]
			set points [centercirclepoints $size $startx $starty $endx $endy $centerx $centery]
			drawskeletonitems $object $drawing $loop $points $helixcmd
		}
		if !$batch {busyincr $object}
	}
	foreach item [$canvas find andtags $drawing _helixname] {
		set name [$canvas itemcget $item -text]
		if [info exists skel(n$name,x)] {
			$canvas coords $item $skel(n$name,x) $skel(n$name,y)
		}
	}
	if {[lsearch $skel(options) flipped]!=-1} {
		flipall $object $drawing
	}
	unset skel
	$canvas zoom $zoom
	setsettings $object $drawing [lindex $settingsline 1]
	$canvas selection redraw
	if !$batch {
		Classy::busy remove $object
		notbusy $object
	}
}

proc drawskeletonitems {object drawing loop points helixcmd} {
	private $object drawinfo canvas
	upvar skel skel
	set ys [expr {$drawinfo($drawing,ys)*2}]
	set lagging 0
	foreach name $loop {
		if [regexp {^S} $name] {
			set lagging 1
			set lname $name
			set items [get drawinfo($drawing,$name,ids) ""]
			set num [llength $items]
			if {$num==0} {
				set lx 0
				set ly 0
			} else {
				if [info exists skel($name,x)] {
					set lx $skel($name,x)
					set ly $skel($name,y)
				} else {
					set pos [expr {(($num-1)/2)*2}]
					set lx [lindex $points $pos]
					incr pos
					set ly [lindex $points $pos]
				}
				set points [lrange $points [expr {$num*2}] end]
			}
		} else {
			if [regexp {'$} $name] continue
			if ![flipped $object $drawing $name] {
				if [info exists skel($name,x)] {
					$helixcmd $object $drawing $name $skel($name,x) $skel($name,y) $skel($name,angle)
					if [info exists skel($name,flip)] {fliphelix $object $drawing $name}
				} else {
					set x1 [list_shift points]
					set y1 [list_shift points]
					set x2 [list_shift points]
					set y2 [list_shift points]
					set angle [sqangle $x1 $y1 $x2 $y2]
					set dist [distance $x1 $y1 $x2 $y2]
					set diff [expr {($ys-$dist)/2}]
					if {$diff!=0} {
						set stx [expr {$x1-($diff/$dist)*($x2-$x1)}]
						set sty [expr {$y1-($diff/$dist)*($y2-$y1)}]
						$helixcmd $object $drawing $name $stx $sty [expr {$angle+90}]
					} else {
						$helixcmd $object $drawing $name $x1 $y1 [expr {$angle+90}]
					}
				}
			}
			if $lagging {
				drawsingle $object $drawing $lname $lx $ly
				set lagging 0
			}
		}
	}
	if $lagging {
		drawsingle $object $drawing $lname $lx $ly
	}
}

proc selectskeleton {} {
	set skeletonfile [Classy::selectfile -title "Select a skeletonfile" -filter *.skl \
		-help select_skeleton -filetypes {{skeleton .skl} {"All files" *}}]
	if {"$skeletonfile"==""} {return ""}
	set f [open $skeletonfile]
	set line [gets $f]
	if ![regexp {^#RnaViz skeleton} $line] {
		if [Classy::yorn "\"$skeletonfile\" is not a valid skeleton file, continue without skeleton?"] {
			set skeletonfile ""
		} else {
			busy remove
			return
		}
	}
	close $f
	return $skeletonfile
}
