proc str1_convert {object file} {
	set f [open $file]
	set line [gets $f]
	if ![string_equal $line "# RnaViz structure"] {
		error "\"$file\" is not a Rnaviz v.1 structure file"
	}
	set result [list header Classy::Canvas-0.1]
	busy $object "Converting file \"$file\" from Rnaviz v.1 format" [file size $file]
	while {![eof $f]} {
		set line [gets $f]
		switch -regexp -- $line {
			"^set font" {
				regexp {set (.*) \[fonttag {(.*)}\]$} $line temp name font
				set fonts($name) $font
				set $name font1
			}
			{^lappend drawinfo[(]drawings[)] } {
				regsub {^lappend drawinfo[(]drawings[)] } $line {} drawings
				foreach el $drawings {
					set [string range $el 1 end] [string range $el 1 end]
				}
			}
			"^array set drawinfo " {
				set len [string length $line]
				set line [string range $line 32 [expr {$len-2}]]
				foreach {option value} [lindex $line 1] {
					if [regexp {^\$d([0-9]+),} $option temp num] {
						set opts [split $option ,]
						if [string_equal [lindex $opts 2] _flip] {
							set flipped(_g$num,[lindex $opts 1]) 1
						} elseif [string_equal [lindex $opts 1] _flip] {
							set flipall(_g$num) 1
						} else {
							set opt [lindex $opts 1]
							if [string_equal $opt itemlist] {
								set opt elementlist
							}
							lappend data(_g$num) $opt $value
						}
					} else {
						if [inlist {papercol paperheight paperrotate papersize paperwidth} $option] {
							lappend data() $option $value
						}
					}
				}
			}
			{^\$c create } {
				break
			}
		}
		busyincr $object [string length $line]
	}
	catch {unset opts}
	catch {unset nums}
	while 1 {
		if ![string_equal	[lindex $line 1] create] {
			break
		}
		switch [lindex $line 2] {
			text {
				set coords [list]
				foreach c [lrange $line 3 4] {
					lappend coords $c
				}
				array set opts {-font {} -anchor c -fill black -justify left -stipple {} -text {} -width 0}
				array set opts [lrange $line 5 end]
				set tags [string trimleft [string trimright $opts(-tags) \"] \"]
				set pos [lsearch -regexp $tags {\$font[0-9]+$}]
				if {$pos != -1} {
					set opts(-font) $fonts([string range [lindex $tags $pos] 1 end])
				}
				foreach {drawing name type part side} $tags {break}
				if [regexp {^\$d([0-9]+)$} $drawing temp num] {
					set newtags _g$num
					if {"$name" == "start"} {
						lappend newtags {_ start} _str
						if [info exists flipall(_g$num)] {
							lappend newtags _flipall
							unset flipall(_g$num)
						}
						set opts(-text) [lreplace $data(_g$num) -1 -1 rnaviz_drawing converted 1]
					} elseif {"$name" == "end"} {
						lappend newtags {_ end} _str
					} elseif {"$name" == "label"} {
						set pos [lsearch -regexp $tags {^l[0-9]+$}]
						if {$pos != -1} {
							set link [lindex $tags $pos]
							regsub ^l $link {} link
							lappend newtags [list _ ? lnk $link]
						}
					} else {
						switch $type {
							c -
							l -
							d {
								if [string_equal $side p3] {
									set name $name'
								}
								regexp {[0-9]+} $part part
								if [string_equal $type l] {incr part}
								lappend newtags [list _ $type $name $part $side] _$name _base _str
								if [info exists flipped(_g$num,$name)] {
									lappend newtags _flipped
									unset flipped(_g$num,$name)
								}
							}
							u -
							s {
								lappend newtags [list _ $type $name] _$name _base _str
							}
							helixname {
								lappend newtags [list _ name $name] _$name _helixname _str
							}
							default {
								lappend newtags _str _text
							}
						}
					}
				} else {
					set newtags _text
				}
				set rtags [list_sub $tags [list_find -glob $tags T*]]
				foreach tag [list_regsub ^T $rtags {}] {
					lappend newtags $tag
				}
				set opt [list $newtags $coords]
				foreach option {-font -anchor -fill -justify -stipple -text -width} {
					lappend opt $opts($option)
				}
				lappend result text $opt
			}
			line {
				set pos 3
				set coords [list]
				while 1 {
					set c [lindex $line $pos]
					if ![isdouble $c] break
					lappend coords $c
					incr pos
				}
				array set opts {-width 1 -arrow none -arrowshape {8 10 3} -capstyle round -fill black -joinstyle round -smooth 0 -splinesteps 12 -stipple {}}
				array set opts [lrange $line $pos end]
				set tags [string trimleft [string trimright $opts(-tags) \"] \"]
				foreach {drawing name type part ns} $tags {break}
				if [regexp {^\$d([0-9]+)$} $drawing temp num] {
					set newtags _g$num
					if {"$name" == "label"} {
						set pos [lsearch -regexp $tags {^l[0-9]+$}]
						if {$pos != -1} {
							set link [lindex $tags $pos]
							regsub ^l $link {} link
							lappend newtags [list _ ? lnk $link]
						}
					} else {
						switch $type {
							dot {
								regexp {[0-9]+} $part part
								if [string_equal $ns nsdot] {
									lappend newtags [list _ dotns $name $part] _$name _dotns _dot _str
								} else {
									lappend newtags [list _ dot $name $part] _$name _dot _str
								}
							}
							background {
								foreach {temp name part} [split $name _] {break}
								regexp {[0-9]+} $part part
								if [regexp ^S $name] {
									if [isint $part] {
										lappend newtags [list _ x2 $name] _background
									} else {
										lappend newtags [list _ x $name] _background
									}
								} else {
									incr part
									lappend newtags [list _ x $name $part] _background
								}
							}
							baseconnect {
								foreach {temp base1 base2} [split $name _] {break}
								lappend newtags [list _ ? bc $base1 $base2] _baseconnect
							}
							box {
								set bases [lrange [split $name _] 1 end]
								set idnt "_ ? box"
								foreach base $bases {lappend idnt $base}
								lappend newtags $idnt _box
								if [inlist $tags inbox] {
									lappend newtags _inbox
								} else {
									lappend newtags _outbox
								}
							}
							default {
								lappend newtags _str
							}
						}
					}
				} else {
					set newtags ""
				}
				set rtags [list_sub $tags [list_find -glob $tags T*]]
				foreach tag $rtags {
					lappend newtags [string range $tag 1 end]
				}
				set opt [list $newtags $coords]
				foreach option {-width -arrow -arrowshape -capstyle -fill -joinstyle -smooth -splinesteps -stipple} {
					lappend opt $opts($option)
				}
				lappend result line $opt
			}
			oval {
				set coords [lrange $line 3 6]
				array set opts {-width 1 -fill {} -outline black -stipple {}}
				array set opts [lrange $line 7 end]
				set tags [string trimleft [string trimright $opts(-tags) \"] \"]
				foreach {drawing name type part ns} $tags {break}
				if [regexp {^\$d([0-9]+)$} $drawing temp num] {
					set newtags _g$num
					if {"$name" == "label"} {
						set pos [lsearch -regexp $tags {^l[0-9]+$}]
						if {$pos != -1} {
							set link [lindex $tags $pos]
							regsub ^l $link {} link
							lappend newtags [list _ ? lnk $link]
						}
					} elseif [string_equal $type background] {
						set list [split $name _]
						switch [lindex $list 0] {
							extra {
								foreach {temp name part} $list {break}
								regexp {[0-9]+} $part part
								if [regexp ^S $name] {
									if [isint $part] {
										lappend newtags [list _ x2 $name] _background _papercol
									} else {
										lappend newtags [list _ x $name] _background _papercol
									}
								} else {
									incr part
									lappend newtags [list _ x $name $part] _background _papercol
								}
							}
							bc {
								foreach {temp base1 base2} $list {break}
								lappend newtags [list _ ? bc $base1 $base2] _baseconnect
							}
						}
					}
				} else {
					set newtags ""
				}
				set rtags [list_sub $tags [list_find -glob $tags T*]]
				foreach tag $rtags {
					lappend newtags [string range $tag 1 end]
				}
				set opt [list $newtags $coords]
				foreach option {-width -fill -outline -stipple} {
					lappend opt $opts($option)
				}
				lappend result oval $opt
			}
			polygon {
				set pos 3
				set coords [list]
				while 1 {
					set c [lindex $line $pos]
					if ![isdouble $c] break
					lappend coords $c
					incr pos
				}
				array set opts {-width 1 -fill {} -outline black -smooth 0 -splinesteps 12 -stipple {}}
				array set opts [lrange $line $pos end]
				set tags [string trimleft [string trimright $opts(-tags) \"] \"]
				foreach {drawing type} $tags {break}
				if [regexp {^\$d([0-9]+)$} $drawing temp num] {
					set newtags _g$num
					if {"$name" == "label"} {
						set pos [lsearch -regexp $tags {^l[0-9]+$}]
						if {$pos != -1} {
							set link [lindex $tags $pos]
							regsub ^l $link {} link
							lappend newtags [list _ ? lnk $link]
						}
					}
				} else {
					set newtags ""
				}
				set rtags [list_sub $tags [list_find -glob $tags T*]]
				foreach tag $rtags {
					lappend newtags [string range $tag 1 end]
				}
				set opt [list $newtags $coords]
				foreach option {-width -fill -outline -smooth -splinesteps -stipple} {
					lappend opt $opts($option)
				}
				lappend result polygon $opt
			}
			rectangle {
				set pos 3
				set coords [list]
				while 1 {
					set c [lindex $line $pos]
					if ![isdouble $c] break
					lappend coords $c
					incr pos
				}
				array set opts {-width 1 -fill {} -outline black -stipple {}}
				array set opts [lrange $line $pos end]
				set tags [string trimleft [string trimright $opts(-tags) \"] \"]
				foreach {drawing type} $tags {break}
				if [regexp {^\$d([0-9]+)$} $drawing temp num] {
					set newtags _g$num
					if {"$name" == "label"} {
						set pos [lsearch -regexp $tags {^l[0-9]+$}]
						if {$pos != -1} {
							set link [lindex $tags $pos]
							regsub ^l $link {} link
							lappend newtags [list _ ? lnk $link]
						}
					}
				} else {
					set newtags ""
				}
				set rtags [list_sub $tags [list_find -glob $tags T*]]
				foreach tag $rtags {
					lappend newtags [string range $tag 1 end]
				}
				set opt [list $newtags $coords]
				foreach option {-width -fill -outline -stipple} {
					lappend opt $opts($option)
				}
				lappend result rectangle $opt
			}
		}
		if [eof $f] break
		busyincr $object [string length $line]
		set line [gets $f]
	}
	set list ""
	set mlist ""
	set prevcur -1
	catch {unset done}
	set size [expr {[llength $result]/2}]
	set list [list_fill $size 0 1]
	while 1 {
		if [regexp {([0-9]+).* ([0-9]+)} $line temp id cur] {
			if {$cur != $prevcur} {
				if [llength $mlist] {
					set list [list_lremove $list $mlist]
					set pos [lsearch $list $prevcur]
					set list [eval {lreplace $list $pos -1} $mlist]
				}
				set prevcur $cur
				set mlist ""
			}
			lappend mlist $id
		}
		if [eof $f] break
		busyincr $object [string length $line]
		set line [gets $f]
	}
	if [llength $mlist] {
		set list [list_lremove $list $mlist]
		set pos [lsearch $list $prevcur]
		set list [eval {lreplace $list $pos -1} $mlist]
	}
	set list [lmath_calc [list_remove $list 0] - 1]
	close $f
	notbusy $object
	lappend result order [list {} $list {}]
	return $result
}

proc str1_removeovals object {
	private $object drawinfo canvas
	set unformatted {}
	set autoconnect {}
	foreach key [array names drawinfo {_*,x2 *}] {
		eval $canvas delete $drawinfo($key)
		catch {unset drawinfo($key)}
		set list [split $key {, }]
		foreach {drawing temp name} $list break
		set okey "$drawing,x $name"
		if [info exists drawinfo($okey)] {
			eval $canvas delete $drawinfo($okey)
			unset drawinfo($okey)
		}
		lappend unformatted $drawing $name
	}
	foreach key [array names drawinfo {_*,x *}] {
		eval $canvas delete $drawinfo($key)
		catch {unset drawinfo($key)}
		set list [split $key {, }]
		lappend autoconnect $list
	}
	foreach item [$canvas find withtag _baseconnect] {
		if [string_equal [$canvas type $item] oval] {
			$canvas delete $item
		} else {
			set tags [$canvas itemcget $item -tags]
			set drawing [lindex $tags 0]
			set name [lindex $tags 1]
			set base1 [lindex $name 4]
			set base2 [lindex $name 7]
			drawconnect $canvas $item [expr {$base1+$drawinfo($drawing,start)}] [expr {$base2+$drawinfo($drawing,start)}]
		}
	}
	foreach list $autoconnect {
		foreach {drawing temp name part} $list break
		if {[llength $list] == 4} {
			drawloop $object $drawing $name $part
		} else {
			drawsingle $object $drawing $name
		}
	}

	foreach {drawing name} $unformatted {
		unformattedconnect $object $drawing $name
	}
}

proc str1_load {object file} {
#putsvars object
	private $object drawinfo canvas
	set data [str1_convert $object $file]
	str_paste $object $data
	str1_removeovals $object
}
